;This takes the list of 10 enemies and works with them -- drawing them,
;updating their position, etc.



BeginEnemyChecklist:

	ld hl, EnemyList
	ld b, 10		;Check for up to ten enemies
	

CheckEnemy:

	push hl
	push bc

	ld a, (hl)
	or a
	jp z, NoEnemy		;Not an enemy


	inc hl			;Get to the next value, part of the X
				;coordinate of the enemy
	push hl
	pop ix			;The position of the enemy's data is
				;stored in IX so that it's easier to access


;First, see if the enemy can be drawn.  Don't do anything to it otherwise.


DrawEnemy:

	ld l, (ix)
	ld h, (ix + 1)		;Get the X coordinate of the enemy.

	ld de, (MapX)		;Find out if the enemy is on-screen,
				;depending on where the player is on the map.
	or a
	sbc hl, de

	ld a, h
	or a
	jr z, _

	cp $FF
	jp nz, NoEnemy		;The enemy is not on screen, so don't do
				;anything with it.

	ld a, l
	cp 240
	jp c, NoEnemy		

	jr DrawEnemyPart2	;The enemy MIGHT be on screen.  We don't
				;know until we look at its Y coordinate.
_

	ld a, l
	cp 108
	jp nc, NoEnemy

DrawEnemyPart2:	

	ld c, l

	ld l, (ix + 2)		;Now check the Y coordinate of the enemy.
	ld h, (ix + 3)

	ld de, (MapY)
	or a
	sbc hl, de

	ld a, h
	or a
	jr z, _


	cp $FF
	jp nz, NoEnemy
	ld a, l
	cp 240
	jp c, NoEnemy
	jr DrawEnemyPart3


_

	ld a, l
	cp 108
	jp nc, NoEnemy


DrawEnemyPart3:

	ld d, c
	ld e, l

;The enemy is onscreen, so we can draw it and mess around with it.


;If the player is jumping or falling, the enemy's Y-coordinate needs to be
;adjusted so that it is drawn in the right location.  Basically it just
;fixes a bug.

	ld a, (JumpLength)
	or a
	jr z, DrawEnemyPlayerNotJumping


	dec e \ dec e \ dec e \ dec e

DrawEnemyPlayerNotJumping:

	ld a, (Falling)
	or a
	jr z, DrawEnemyPlayerNotFalling

	inc e \ inc e \ inc e \ inc e

DrawEnemyPlayerNotFalling:

;Now we can draw the sprite.  Get to the correct sprite.

	ld b, (ix-1)			;Which Gruzzle is this?
	ld hl, EnemySprites -256 -2
_
	inc h
	djnz -_


DrawEnemySprite:


#ifdef DOBOSS


	ld a, (ix-1)
	cp 6
	jr nz, NoDrawBossBullet

DrawBossBullet:

	push ix
	ld b, 8
	ld hl, BossSprite+256-2
	call ClipSprite_Mask+2
	jr CheckCollisionWithGrizzle



NoDrawBossBullet:

	ld a, (ix-1)
	cp 5
	jr z, DoDrawBoss
	
	ld a, (BossHitsLeft)
	inc a
	jr nz, NoDrawBoss

DoDrawBoss:
	push ix


	call DrawBoss



	ld a, (BossHitsLeft)
	inc a
	jr nz, CheckCollisionWithGrizzle
	pop ix
	jp NoEnemy


NoDrawBoss:

#endif




	inc (ix+5)	;Increase the frame (Each Gruzzle has 4 frames
			;of animation)

	ld a, (ix+5)	;Draw the correct sprite depending on which
			;frame of animation the gruzzle is currently at
	ld b, a
	srl b \ srl b
	jr z, FinallyDrawEnemySprite

	cp 16
	jr c, EnemyAddB

	cp 24
	jr c, _

	
	ld (ix+5), 0
	jr FinallyDrawEnemySprite
_
	cp 20
	jr c, _

	ld b, 2
	jr EnemyAddB

_

	ld b, 1


EnemyAddB:

	ld a, 32
	call Add_A_To_HL
	djnz EnemyAddB

FinallyDrawEnemySprite:

;Clip_Sprite_Mask draws a sprite only 8 pixels wide, so two 8x16 sprites
;need to be drawn.

	push ix
	push de
	push hl
	call ClipSprite_Mask
	pop hl

	ld de, 128
	add hl, de
	pop de
	ld a, d
	add a, 8
	ld d, a


	
	call ClipSprite_Mask




;The enemy was drawn and is onscreen, so we can move it.


CheckCollisionWithGrizzle:

	pop ix

	call CheckEnemyCollision



#ifdef DOBOSS

;Bullets from the boss only move up

	ld a, (ix-1)
	cp 6
	jr nz, EnemyNotBullet

MoveBossBullet:



;Move the bullet up four more pixels.

	ld l, (ix+2)
	ld h, (ix+3)
	dec hl
	dec hl
	dec hl
	dec hl

	ld (ix+3),h
	ld (ix+2), l



;If the bullet has gone up far enough, shoot a new bullet

	ld de, 17*16
	or a
	sbc hl, de
	jr nc, ExitBulletRoutine

;ChangeBullet

	ld hl, BossGruzzle+7
	ld de, EnemyList+7
	ld bc, 7
	ldir

;Random X Position depending on where the Boss is

	ld l, (ix-7)
	ld h, (ix-6)	;X Position of the Boss

	ld de, -13
	add hl, de

	ld a, r
	and %00000110
	rrca
	or a
	jr z, NewBossBullet

	ld de, 7
	ld b, a
_
	add hl, de
	djnz -_

NewBossBullet:

	ld (ix+0), l
	ld (ix+1), h

ExitBulletRoutine:

	pop bc
	pop hl

	jp NoBulletsEither

#endif


EnemyNotBullet:


	ld a, (ix +4)	;Points to the data of which direction the
			;enemy should move, as well as how many pixels.

			;0-15 means the enemy moves to the left.
			;16-31 means the enemy moves to the right.

	dec a	;ChangeDirection
	jp z, GruzzleChangeDirection

	cp 16
	jp z, GruzzleChangeDirection

;If there is no need to change direction,
;simply decrease the number of times the gruzzle needs to move in the current
;direction.

	dec a
	ld (ix+4), a

DontChangeDirection:

	ld e, (ix)
	ld d, (ix+1)	;The X coordinate.  Gruzzles can fall down,
			;but they NEVER move up.  Thus they move
			;only left or right unless they fall.

	cp 16
	jr nc, MoveGruzzleRight

MoveGruzzleLeft:

	dec de

	ld a, (ix-1)
	cp 5
	jr nz, CheckCollisionGruzzle
	dec de

	jr CheckCollisionGruzzle


MoveGruzzleRight:


	inc de

	ld a, (ix-1)
	cp 5
	jr nz, CheckCollisionGruzzle
	inc de



CheckCollisionGruzzle:


;This function does two things.

;First, it sees if the Gruzzle is on something solid -- if not,
;the Gruzzle is falling.

;Second, it sees if the Gruzzle is hitting a wall when moving
;left and right.



	ld (ix), e
	ld (ix + 1), d		;Save the updated X coordinate -- the Gruzzle
				;has moved left/right.








	ld l, (ix + 2)
	ld h, (ix + 3)		;Get the Y coordinate.  DE
				;still contains the X Coordinate.

	push de

	call GetTilePlayerIsOn	;Get the tile on the tilemap where the
				;Gruzzle is located.  We use this
				;value only if is not possible for the
				;enemy to fall.
	
	pop de			;We still need the Gruzzle's new X coordinate

;Check first to see if the enemy needs to fall

	ld a,(ix)		;If the Gruzzle is not squarely lined up
				;with the tile beneath, it does not need 
				;to fall.

;I don't remember why I had this code here,
;but it's necessary to ensure that the enemy will fall when it needs
;to.  Basically, it forces the Gruzzle to line up with a tile
;when it's close enough.

;I do know that part of the reason I have this code is to account for the
;fact that the Gruzzle is constantly on the move.

	and %00001111
	cp 15			
	jr z, AdjustFall15
	cp 1
	jr z, AdjustFall1
	or a
	jr z, DoneAdjustFall

	jr EnemyIsNotFalling

AdjustFall15:

	inc de
	jr DoneAdjustFall

AdjustFall1:

	dec de

DoneAdjustFall:

;The Gruzzle is lined up, is there a solid tile beneath it, or does
;it need to fall?

;The gruzzle will not fall if it is a boss.

	ld a, (ix-1)
	cp 5
	jr z, EnemyIsNotFalling

	ld l, (ix+2)			;Previously, HL pointed to the
					;tile the enemy was located on.
					;But we need to refind the tile,
					;since now there is a possibility
					;for the enemy to fall.
	ld h, (ix+3)
	
	ld (ForEnemyFall), de		;Save this value for later.
					;If the enemy is falling, we want
					;it to fall "Straight Down", as
					;opposed to the fact that a few lines
					;of code back it moved left/right

	call GetTilePlayerIsOn


	push hl				;The tile points to the tile
					;directly BEHIND the enemy.
					;Get the tile BELOW the enemy.
	ld de, (MapWidth)
	add hl, de
	ld a, (hl)
	pop hl

;Check for "SOLID TILES"


	cp 192				
	jr nc, EnemyIsFalling
	and %00001100	
	cp %00001100
	jr z, EnemyIsNotFalling

EnemyIsFalling:

	ld l, (ix+2)		;Adjust the Gruzzle's Y coordinate, since
				;it is falling.
	ld h, (ix+3)

	ld de, 4
	add hl, de

	ld (ix+2), l
	ld (ix+3), h

	ld hl, (ForEnemyFall)	;The enemy moved to the right or the left,
				;but we want the enemy to fall "straight down."
	ld (ix), l
	ld (ix+1), h



	jr NoEnemy		;We are done with this enemy
	

EnemyIsNotFalling:


#ifdef DOBOSS

	ld a, (ix-1)
	cp 5
	jr z, NoEnemy

#endif



;See if the enemy is bumping into a tile by moving left or right.
;HL points to the tile BEHIND the enemy.


;If the Gruzzle is moving RIGHT, check the tile on the Gruzzle's right.
;Otherwise, check the tile on the Gruzzle's left.

	ld a, (ix + 4)
	cp 16
	jr c, _

	inc hl
	jr ++_
_
	dec hl
_

	ld a, (hl)
	cp 176
	jr nc, NoEnemy
	and %00001100
	cp %00001100
	jr nz, NoEnemy	;Don't change the Gruzzle's direction, because
			;it's not bumping into a solid tile.

ChangeDirectionCollision:

;This changes the direction because a Gruzzle bumped into a tile.

	ld a, (ix + 4)
	xor %00010000
	or %00001111
	ld (ix + 4), a

	jr NoEnemy


GruzzleChangeDirection:

	;The Gruzzle needs to "chase" the player.  After moving 16 pixels
	;in a direction, calculate to see if it needs to change the
	;direction to chase the player.

	ld e, (ix)
	ld d, (ix + 1)

	ld hl, (MapX)
	ld bc, 48
	add hl, bc
	or a
	sbc hl, de
	jr c, GrizzleMoveLeft

GrizzleMoveRight:

	ld (ix+4), 31
	jr NoEnemy

GrizzleMoveLeft:

	ld (ix+4), 15


;;;;;;;;;;;;;;;;;;;;;
;
; DONE DOING EVERYTHING
; WITH ONE ENEMY.  MOVE
; TO THE NEXT ENEMY.
;
;;;;;;;;;;;;;;;;;;;;;;
	

NoEnemy:

	pop bc
	pop hl

#ifdef DOBOSS

	ld a, (hl)
	cp 6
	jr nz, NoBulletsEither

	push hl
	push bc
	jp MoveBossBullet

#endif


NoBulletsEither:



	ld de, 7	;Skip 7 bytes of data to get to the next enemy.
	add hl, de
	dec b
	jp nz, CheckEnemy

;Done working with the 10 enemies

	ret



;;;;;;;;;;;;;;;;;;;;;;
;
; AFTER THE MAP HAS
; BEEN DECOMPRESSED,
; FIND WHERE THE ENEMIES
; ARE AND STORE THEIR DATA,
; SUCH AS X AND Y COORDINATES
;
;;;;;;;;;;;;;;;;;;;;;;



BeginningGameGetEnemies:


	.db 0

	ld (spbackup), sp  ;We need to use SP, so save its value


	ld sp, MapLocation	;SP is the map tile

	ld hl, 0	;MapX, start at X = 0
	ld de, 8	;MapY, start at Y = 0

	ld a, (MapWidth)
	ld c, a
	ld a, (MapHeight)
	ld b, a
	ld ix, EnemyList   ;Where the data for enemies is to be stored

CheckRowForEnemy:

	ex (sp), hl

	ld a, l		;A = (HL)
	cp 28		

;If C, the tile is NOT an enemy, so we do not need to store data.

	jr c, GotoNextCheckRowForEnemy
	cp 32
	jr c, CreateEnemy


;The tile is not an enemy.  It may be a tree or a rock, but not a Gruzzle
	
GotoNextCheckRowForEnemy:



	ex (sp), hl

;Enemy X and Y coordinates are stored in pixels, so add 16 pixels.


	ld a, 16
	add a, e
	ld e, a
	jr nc, _

	inc d
_

	dec c		;One less tile in the current row to check

	inc sp		;Go to the next tile in the tilemap

	jr nz, CheckRowForEnemy

	ld de, 16	;We are done searching the CURRENT row for
			;enemies.  Check the next row.

	add hl, de
	ld de, 8
	ld a, (MapWidth)
	ld c, a
	djnz CheckRowForEnemy
	ld sp, (spBackup)
	ret
	
;The tile is a Gruzzle!  Store the correct data.
	
CreateEnemy:

	sub 27		;Enemy number, from 1 to 4.
	ld (ix), a	;The first byte of data is the enemy ID.

	ex (sp), hl

	ld (ix + 1), e	;Store the enemy's X coordinate
	ld (ix + 2), d
	
	ld (ix + 3), l	;Store the enemy's Y coordinate
	ld (ix + 4), h

	ld (ix+5), 1	;Starting direction
	ld (ix+6), 0	;Frame #0 out of 4 frames for animation
	ex (sp), hl

	ld l, h

	inc ix
	inc ix
	inc ix
	inc ix
	inc ix
	inc ix
	inc ix

	jr GotoNextCheckRowForEnemy

CheckEnemyCollision:

;Has the Gruzzle collided with the player?
;The Gruzzle will almost NEVER collide with the player if they are not both
;on the ground.
;
;So if their Y coordinates don't match, there's no collision.

CheckEnemyYCollision:

	ld e, (ix + 2)
	ld d, (ix + 3)

;Check for collision

	ld hl, (MapY)	
	ld bc, 25	
	add hl, bc		;Now HL = the player's Y coordinate

	or a
	sbc hl, de
	ret c

EnemyIsAbovePlayer:

	ld a, h
	or a
	ret nz

	ld a, l
	cp 16
	ret nc




CheckEnemyXCollision:

	ld e, (ix)
	ld d, (ix + 1)

	ld hl, (MapX)
	ld bc, 48
	add hl, bc
	or a
	sbc hl, de
	ld a, l
	ld (ForDeath), a
	jr c, EnemyIsOnRightOfDevax

EnemyIsToTheLeftOfDevax


	
	cp 48
	ret nc	;Too far for collision or nuke


	ld a, (LastDirection)
	cp Left
	call z, CheckNuke

;The enemy is on the left, but the player is facing right, so the player
;dropping a nuke isn't aiming for the gruzzle

	ld a, l

	cp 16
	jr c, EnemyCollided

;The Gruzzle has not collided with the player.

	ret

EnemyIsOnRightOfDevax:

	cp -32
	ret c	;Too far for collision or nuke

	ld a, (LastDirection)
	cp Right
	call z, CheckNuke

	ld a, l
	cp -4
	jr nc, EnemyCollided
	ret

EnemyCollided:

;If the player has reached the door to end the game, the player is
;invincible.

	ld a, (ClearGame)
	or a
	ret nz

;If the player has recently already collided with a gruzzle, he is
;temporarily invinciple.  Check for this.

	ld a, (HitGruzzleTimer)
	or a
	ret nz

;Does the player have any shields left?

	ld a, (ShieldsLeft)
	or a
	jr z, PlayerCanDieByGruzzlesHand

PlayerLosesAShield:



	call GivePlayerWarning


	ret


PlayerCanDieByGruzzlesHand:

;If the player is out of shields, he dies.

	ld a, 87
	ld (DeathWish), a

	ret









CheckNuke:

;Check to see if the player successfully hits a gruzzle with a nuke.

;The player cannot drop a nuke if he is running

	ld a, $FE
	out (1), a
	ex (sp), hl
	ex (sp), hl
	in a, (1)
	inc a
	ret nz

;After running, it takes time for the player to be able to use his weapon.

	ld a, (ReadyWeaponTimer)
	or a
	jr z, CanFireNuke

	dec a
	ld (ReadyWeaponTimer), a

	ret

CanFireNuke:

;Is the player pressing the "ALPHA" key to fire a nuke?

	ld a, $DF
	out (1), a
	ex (sp), hl
	ex (sp), hl
	in a, (1)


	bit 7, a
	ret nz

;The player can't destory a "boss missile" with a nuke

	ld a, (ix-1)
	cp 6
	ret z

;The player can't fire a nuke if the gruzzle is not touching the ground

	ld a, (ix+2)
	and %00001111

	ret nz

;If the player is out of nukes, he can't fire a nuke!

	ld a, (Nukes)
	or a
	ret z
_

	ld a, (DontCalculateNuke)
	or a
	ret nz
	ld a, 1
	ld (DontCalculateNuke), a

	ld (ix+5), 0
	push ix


	di

;Redraw the screen.  It will be saved to "savesscreen" so that it can be
;redrawn to show the animation of the nuke falling on the gruzzle.

	call updatemapfull		;Redraws the map

	call BeginEnemyCheckList	;Redraws the enemy Gruzzle
	
	ld a, 1
	ld (NukeY), a	;The Y coordinate of the nuke


	call DrawCharacter		;Redraws the character
	call DrawHud			;Redraws the hud

	ld hl, plotsscreen
	ld de, savesscreen
	ld bc, 768
	ldir

	pop ix

	inc sp		;RET to the correct location
	inc sp

	call DrawAttack		;Draws the animation of the nuke
				;landing on the gruzzle


#ifdef DOBOSS



	ld a, (BossHitsLeft)
	inc a
	jr nz, NoKillBossAnimation

KillBossAnimation:

	xor a
	ld (EnemyList+7), a
	ld a, 5
	ld (EnemyList),a


	ld hl, (MapX)
	
	inc hl
	inc hl
	inc hl

	ld (MapX), hl

	ld b, 96
_
	push bc
	push ix

	ld hl, (MapX)
	bit 0, b
	jr z, _
	
	ld de, -6
	add hl, de
	jr ++_
_
	ld de, 6
	add hl, de
_

	ld (MapX), hl

	call updatemapfull
	
	call BeginEnemyCheckList

	pop ix
	pop af
	push af
	push ix
	
	cp 60
	jr nc, NoDrawBossExplosions
	call DrawExplosion
	call DrawExplosion
	call DrawExplosion

NoDrawBossExplosions

	call DrawCharacter
	call SafeCopy

	ld b, 0
_
	djnz -_
_
	djnz -_
	
	pop ix
	pop bc
	djnz -----_

	ld b, 64

	
_
	push bc
	call updatemapfull
	ld hl, (EnemyList+3)
	inc hl
	ld (EnemyList+3), hl
	
	call BeginEnemyCheckList
	call DrawCharacter
	call SafeCopy
	pop bc
	djnz -_

	xor a
	ld (EnemyList),a
	ld (DeathWish),a
	ld (QuestionsLeft), a

	ld hl, LevelCompleteText
	call CreateCustomHudString
	


NoKillBossAnimation:




#endif



	xor a
	ld (DontCalculateNuke), a
	ld (HitGruzzleTimer), a

	ld de, 20		;20 points for hitting a gruzzle with a nuke!
	call IncreaseScore

	ld a, (ForDeath)

;Is the player too close to the nuke?

	bit 7, a
	jr nz, _

	neg
	add a, 16
_

	cp -12
	ret c

NukeDeathWish:

	ld a, (ShieldsLeft)
	or a
	jr z, _


	call GivePlayerWarning
	ret
_

;If the player is too close to a nuke, he dies.

	ld a, 87
	ld (DeathWish), a

	ret


DrawBoss:

	ld a, d
	sub 8
	ld d, a
	ld a, e
	sub 16
	ld e, a

	ld (ForBossDeath), de
	ld hl, BossSprite-2


	push de
	push hl

	ld b, 32
	call ClipSprite_Mask+2

	ld b, 3

_
	pop hl
	ld de, 64
	add hl, de
	pop de
	ld a, d
	add a, 8
	ld d, a

	push de
	push hl
	
	push bc
	ld b, 32
	call ClipSprite_Mask+2
	pop bc
	djnz -_
	
	pop hl
	pop de


	ret


DrawExplosion:

	ld de, (ForBossDeath)


	ld a, r
	and %01111100
	rrca \ rrca \ rrca
	add a, e
	ld e, a

	ld a, r
	and %00011111
	 

	add a, d
	ld b, 16

	ld ix, Sprites + (32*31)
	ld iy, Sprites + (32*32) 

	call DoubleMaskedSpriteMapPage1
	ret


GivePlayerWarning:

	dec a
	ld (ShieldsLeft), a

	ld a, 32		;The player is temporarily invincible,
				;giving the player a chance to run away
				;from the gruzzle that it hit or the hazard
				;that it hit

	ld (HitGruzzleTimer), a


	ret