

InGameGetKey:


;If the player is moving, he cannot fire his weapon.  Furthermore,
;it takes time after the player stops moving before he can fire his weapon.

	ld a, $FE
	out (1), a
	ex (sp), hl
	ex (sp), hl
	in a, (1)
	inc a
	jr z, _


	ld a, 10
	ld (ReadyWeaponTimer), a

_


;The player moves up or down 1 tile of a ladder, in increments of 4 pixels.
;If ElevatorStuff+2 is not 0, the player has not finshed moving up/down
;1 ladder tile.  Thus the player needs to finish


	ld a, (ElevatorStuff+2)
	or a
	jp nz, MoveElevatorFinishTheJob

;Does the player want to display different information on the HUD?

	ld a, $BF
	out (1), a
	ex (sp), hl
	ex (sp), hl
	in a, (1)
	bit 4, a
	jr nz, NoChangeHud

;If the player has not released the "Y=" key, don't change the HUD twice.
;This way the player changes the HUD only once each time he presses the key.
	
	ld a, (KeyHeld)
	or a
	jr nz, NoReleasedHud

	inc a
	ld (KeyHeld), a

	ld a, (Hud_Display)
	inc a
	and %00000011

	ld (Hud_Display), a
	jr NoReleasedHud

NoChangeHud:
	
	xor a
	ld (KeyHeld), a




NoReleasedHud:

;Done with the HUD-changine routine.

;Check for climbing up or down a ladder.



	ld hl, (MapY)
	ld de, (MapX)


	call GetTilePlayerIsOn

;HL is the tile on the upper-left hand corner of the screen.

	
	ld a, $FE
	out (1), a
	ex (sp), hl
	ex (sp), hl
	in a, (1)

;If the player is moving left or right,
;the player is NOT climbing up or down a ladder.

	bit 1, a
	jr z, NoElevatorMove
	bit 2, a
	jr z, NoElevatorMove

	bit 0, a
	jr z, CheckMoveElevatorDOwn

	bit 3, a
	jr nz, NoElevatorMove

CheckMoveElevatorUp:


	ld de, -4	;Up 4 pixels
	ld b,2		;After calculating the location of the tile
			;on the upper-left hand corner of the screen,
			;move down 2 rows for seeing if there's a ladder
			;for the player to climb UP

	jr GetToCheckForLadderTile





CheckMoveElevatorDown:

		
	ld de, 4	;Down 4 pixels
	ld b,3		;After calculating the location of the tile
			;on the upper-left hand corner of the screen,
			;move down 3 rows for seeing if there's a ladder
			;for the player to climb DOWN

GetToCheckForLadderTile:

	push de
	ld de, (MapWidth)
_
	add hl, de
	djnz -_
	inc hl
	inc hl
	inc hl		;Move to the right 3 tiles to get to tile in the middle
			;of the screen.  Otherwise, HL will not correctly
			;point to a tile that could possibly be a ladder
			;above or below the player.

	pop de
	ld a, (hl)
	cp 191		;191 is the ID for the ladder tile.

	jr nz, NoElevatorMove

	ld a, 1
	ld (Climbing), a	;The player is climbing up / down the ladder


MoveElevatorDown:

	ld a, (MapX)	;Center the ladder onto the middle of the screen.
	and %11110000
	or %00001000
	ld (MapX), a

	ld a, (MapY)
	and %11110000
	or %00001000
	ld (MapY), a

	ld (ElevatorStuff), de		;The player moves up/down a ladder
					;4 times, 4 pixels at a time.
					;Save this data.
	ld a, 4
	ld (ElevatorStuff+2), a

MoveElevatorFinishTheJob:

;If an player is still moving up or down a ladder, continue the process.

	xor a
	ld (JumpLength), a	;The player is NOT jumping
	ld (Falling), a		;The player is NOT falling

	ld de, (ElevatorStuff)

	ld hl, (MapY)
	add hl, de
	ld (MapY), hl

	ld hl,ElevatorStuff+2
	dec (hl)

	call umf		;Draw the tilemap

	ret 			;Don't do anything else until
				;the player is done moving up / down
				;the ladder

NoMoveElevatorDown:

NoElevatorMove:


	xor a
	ld (Climbing), a	;The player is NOT climbing a ladder





CheckForOverlapPowerup:

;Has the player hit a powerup?


	ld hl, (MapY)
	ld de, (MapX)


	call GetTilePlayerIsOn



	ld de, (MapWidth)
	add hl, de
	add hl, de
	inc hl
	inc hl
	inc hl

	ld (PlayerTile), hl	;Save the tile directly BEHIND the player.

	push hl
	pop ix			;Save the pointer to this tile in IX,
				;in order to see if the player has
				;touched a powerup
	
	ld a, (ix)
	call CheckForPowerUp

;Is the player moving left or right?

	ld a, $FE
	out (1), a

	ex (sp), hl
	ex (sp), hl
	
	in a, (1)


DGF:

	and %00000110
	cp %00000100
	jp z, MoveCharacterLeft
	cp %00000010
	jp z, MoveCharacterRight

;If the player is not moving left or right, do not show an animation
;of the player running.

	xor a
	ld (PlayerFrame), a

EHG:



TestForFallDownOrJumpUp:


TFFDOJU2:

;After the player has moved left/right, redraw the tilemap.
;Data from plotsscreen is needed to check for up/down collisions.

	call umf	



_

	ld a, (Falling)		;If the player is falling,
				;he cannot jump.
	or a
	jr nz, NotJumpingUp


;Check to see if the player wants to jump

	ld a, $BF
	out (1), a
	ex (sp), hl
	ex (sp), hl
	in a, (1)


	bit 5, a

	jr nz, NotJumpingUp


	


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; PLAYER IS JUMPING IN THE AIR
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Jumping:
	


	ld hl, plotsscreen + (23 * 12 + 4) 	;Check this area
						;of plotsscreen for
						;collision detection
						;purposes
	ld (UpOrDown), hl

	ld a, (JumpLength)		;The player can only
					;jump so high.
	dec a
	jr nz, _			;If the player can still go higher,
					;the player doesn't have to fall.
	ld (JumpLength), a
	jr NotJumpingUp			;The player cannot go any higher.
					;He starts to fall.

_

;If JumpLength was previously 0, the player was on the ground.  Then
;the player wanted to jump, so decreasing A=(JumpLength) means A=$FF.
;If JumpLength was NOT 0, do not reset how far the player can jump.

	cp $FF
	jr nz, _

	ld a, 12	;The player can move up 12 times (4 pixels each
			;time) during the jump.  Thus a player
			;can only jump 3 tiles high.

_
	ld (JumpLength), a
	
	ld b, a



;Do not check for collision if the player's head is not touching the bottom
;of the tile above him.

	ld a, (Mapy)
	and %00001111
	cp 8
	jr nz, DevaxHasNotHitCeiling

;Do not check for collision if the player is jumping as a result
;of touching a hazard

	ld a, (DangerJumpTimer)
	or a
	jr nz, DevaxHasNotHitCeiling

;The player's head is touching the bottom of the tile above him.  Check
;for collision.


	call CheckAboveForCollision
	jr c, DevaxHasHitCeiling


DevaxHasNotHitCeiling:

;Move up 4 pixels and leave.

	ld hl, (MapY)
	ld de, -4
	add hl, de
	ld (MapY), hl

	ret

DevaxHasHitCeiling:

;If the player hit the ceiling, he can't go any higher.  Instead he falls.

	ld a, 1
	ld (JumpLength), a

	ret


NotJumpingUp:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; PLAYER IS JUST PLAIN FALLING
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	

NotJumpingAtAll:

;A special section of code for checking for the presence of a ladder.
;This fixes a bug.

;;;;;;;;;;;;;;;;;;;
;
; IF YOU MODIFY THE LADDER TILE, YOU MUST
; MODIFY THIS SECTION OF CODE,
; AND VICE-VERSA.
;
;;;;;;;;;;;;;;;;;;;

	ld a, (plotsscreen + (41*12+5))
	cp %01011111
	jr nz, NoLadderFix
	xor a
	ld (Falling), a
	ret

NoLadderFix:

;The player's jump is not offically finished until he is centered with
;the tile behind him.  In other words, a player must jump 1, 2 or 3 whole
;tiles.

;Thus, even if the player has released the 2nd key, he may need to finish
;his jump.

	ld a, (JumpLength)	;If the jump is done, it's done.
	or a
	jr z, _

;Otherwise, has the player jumped a whole tile?

	ld a, (Mapy)
	and %00001111
	cp 8
	jr nz, Jumping		;Nope!  Finish moving the length of a tile.

;If the player has hit something dangerous, the player
;continues to "jump" with no collision check


	ld a, (DangerJumpTimer)
	or a
	jr nz, Jumping




;If the player is not pressing 2nd and has finished moving up a whole tile,
;don't jump anymore.

	xor a
	ld (JumpLength), a	

_

;If the player has hit something dangerous, the player
;continues to "jump" with no collision check


	ld a, (DangerJumpTimer)
	or a
	jr nz, PlayerIsFalling

;Use plotsscreen to check for collision.  If the player lands on a tile,
;he cannot fall anymore

	ld hl, plotsscreen + (40 * 12 + 4)
	ld (UpOrDown), hl

;Do not check for collision if the player's feet are not touching the very top
;of the tile below him.


	ld a, (MapY)
	and %00001111
	cp 8
	jr nz, PlayerIsFalling
	call CheckBelowForCollision
	jr c, PlayerHasLanded



PlayerIsFalling:


	ld a, 1
	ld (Falling), a

	ld hl, (MapY)
	ld de, 4
	add hl, de
	ld (MapY), hl


	jp DoneTestingGoingUpOrDown

PlayerHasLanded

	xor a
	ld (Falling), a


	ret







CheckBelowForCollision:
CheckAboveForCollision:
CheckDownUpCollision:


;This is a special collision routine that checks plotsscreen.


	ld a, (MapX)
	and %00001111

;If the player is centered on a tile, do a special special collision test.

	cp 8
	jr z, SpecialTestFallDown


	ld e, a
	ld d, 0


	ld a, (LastDirection)
	
	or a
	jr nz, TestFallDownMovingLeft


TestFallDownMovingRight:
	
;Test to see if pixel number 15 on the tile is black	


	ld hl, ANDerRight
	add hl, de
	ld c, (hl)
	ld hl, (UpOrDown)

	ld a, e
	cp 8

	jr nc, DontDecreaseHLDown
	or a
	jr z, DontDecreaseHLDown

	dec hl
	jr DontDecreaseHLDown



TestFallDownMovingLeft:

;Test to see if pixel number 0 on the tile is black

	ld hl, ANDerRight

	add hl, de
	ld c, (hl)
	ld hl, (UpOrDown)
	inc hl

	ld a, e
	or a
	jr z, _
	cp 8
	jr c, DontDecreaseHLDown
_

	inc hl




DontDecreaseHLDown:


	
	ld a, c
	and (hl)
	jr nz, _

	ld a, (LastDirection)
	or a
	jr z, R3
L33:
	dec hl
	dec hl
	jr LRR

R3:

	inc hl
	inc hl

LRR:
	ld a, c
	and (hl)
	jr z, NoCollisionDownUp
_
	scf
	ret
	

NoCollisionDownUp:

	or a
	ret


	




SpecialTestFallDown:


	ld hl, (UpOrDown)

;Test to see if pixel number 15 on the tile is black

	inc hl
	bit 7, (hl)

	jr z, _
	scf
	ret
_
	or a
	ret z


DoneTestingGoingUpOrDown:

	ret
	



DoNotAllowLeftRight:
	








MoveCharacterLeft:

	call IncreasePlayerFrame	;Animate the player running.


	ld a, left
	ld (lastDirection), a
	
	ld de,(mapx)

	ld a, e
	and %00001111
	cp 12
	jr nc, NoCollisionLeft

	ld hl, (playertile)
	dec hl
	ld (PlayerTile), hl

	ld a, (hl)
	cp 176
	jr nc, NoCollisionLeft
	cp 79
	jr z, NoCollisionLeft

	and %00001111
	cp 12
	jr c, NoCollisionLeft
	ld a, e
	and %11110000
	or 8
	ld (MapX), a
	jp TestForFallDownOrJumpUp



NoCollisionLeft:

	ld hl, (mapX)
	ld de, -4

	add hl, de
_
	ld (mapx), hl

	jp TestForFallDownOrJumpUp


MoveCharacterRight:

	call IncreasePlayerFrame


	ld a, right
	ld (lastDirection), a
	

	ld de, (MapX)

	ld a, e
	and %00001111
	cp 4
	jr c, NoCollisionRight


	ld hl, (playertile)
	inc hl
	ld (PlayerTile), hl
	ld a, (hl)
	cp 176
	jr nc, NoCollisionRight
	cp 79
	jr z, NoCollisionRight
	and %00001111
	
	cp 12
	jr c, NoCollisionRight
	
	ld a, e
	and %11110000
	or 8
	ld (MapX), a

	jp TestForFallDownOrJumpUp




NoCollisionRight:

	ld hl, (MapX)
	ld de, 4
	add hl, de	

	ld (mapx), hl


	jp TestForFallDownOrJumpUp




		
ANDERRight:

;This data will mask out the appropriate pixel on plotsscreen when we
;need to test to see if a player hits the ceiling or lands on the ground.

.db %10000000
.db %00000001
.db %00000010
.db %00000100
.db %00001000
.db %00010000
.db %00100000
.db %01000000
.db %10000000
.db %00000001
.db %00000010
.db %00000100
.db %00001000
.db %00010000
.db %00100000
.db %01000000


IncreasePlayerFrame:

	ld a, (PlayerFrame)
	inc a
	and %00001111
	ld (PlayerFrame), a
	ret
	

