CheckForPowerup:



#ifdef DOBOSS

	cp 145
	jr nz, CheckForExitDoor

	ld (OP6 + 9), a

	ld a, (MapY)
	and %00001111
	
	jr nz, NoExtendFallDownBossLevel

	ld a, (BossLevelFallDown)
	or a
	jr z, NoExtendFallDownBossLevel

	dec a
	ld (BossLevelFallDown), a
	jr nz, NoEndBossLevelFallDown

	ld hl, MapLocation + (24*16)+3
	ld a, 86

	ld (hl), a
	inc hl
	ld (hl), a


	ld bc, $FFFF
_
	dec bc
	ex (sp), hl
	ex (sp), hl
	ld a, b
	or c
	jr nz, -_

	ld a, 1
	ld (FirstPartOfBoss), a

	ld hl, 24*16
	ld (MapY), hl

NoEndBossLevelFallDown:


	ld hl, (MapY)
	ld de, -16
	add hl, de
	ld (MapY), hl

StoreBossAsEnemy:

;Clear all enemies to make room for boss and bullets

	ld hl, BossGruzzle
	ld de, EnemyList
	ld bc, 14
	ldir

NoExtendFallDownBossLevel:

	ld a, (OP6 + 9)
	ret



BossGruzzle:

.db 5

;X Coordinate
.dw 96
;Y Coordinate
.dw 25*16
.db 1,0

.db 6
.dw 99-16
.dw 24*16
.db 1,0


#endif



CheckForExitDoor

;IX contains the tile in question, the tile that the player
;has run into

;The exit door?

	cp 7
	jr nz, NoDoor

;If the player hasn't collected all of the questions,
;he cannot pass through the door.

	ld a, (QuestionsLeft)
	or a
	ret nz

;If the player HAS collected all of the questions and HAS
;touched the door, he doesn't need to do so a second time.

	ld a, (ClearGame)
	or a
	ret nz

;This will start the sequence that blackens the game and moves
;the player to the next level.
	
	ld a, 65
	ld (ClearGame), a
	ret

NoDoor:

;Has the player fallen into a hazard such as water, a pit
;or lava?

	cp 2
	jr z, FallenIntoPit

	cp 32
	jr nc, NoDie

	and %00001100
	cp %00001000
	jr nz, NoDie

FallenIntoPit:

	ld a, (ShieldsLeft)
	or a
	jr z, _


	call GivePlayerWarning

	ld a, 12
	ld (JumpLength), a
	ld a, 20
	ld (DangerJumpTimer), a

	ret
_
	ld a, 89
	ld (DeathWish), a
	ret


NoDie:

;Anything else below tile 192 is not a powerup.

	cp 192
	ret c

CheckBooks:

	cp 255
	jr nz, NotBooks

IsBooks:


	ld de, 5
	call IncreaseScore

	;Look at tiles
	;above the book until a tile that is not a book
	;is found.  This tile will replace the book and
	;restore the background.

	push hl
	push ix

	pop hl
	push hl

	ld de, (MapWidth)

_
	or a
	sbc hl, de
	ld a, (hl)
	cp 255
	jr z, -_

	pop hl
	ld (hl), a
	pop hl

;One less book.  If all books are collected, the player
;gets an extra nuke, extra points and an extra shield
;the next game.

	ld iy, BooksLeft
	dec (iy)

	jr nz, NoBookBonus

BookBonus:

	ld de, 500
	call IncreaseScore
	call IncreaseNukes
	call IncreaseShieldsNextGame

;Let the player know he got the book bonus

	ld hl, BookBonusText
	call CreateCustomHudString

NoBookBonus:

	ld iy, flags
	ret


NotBooks:
CheckForNukes:

	cp 253
	jr nz, CheckForQuestionMark

;The player gets an extra nuke.  Get the tile next to the
;nuke and use it to restore the background behind the nuke

	call ReplaceBackground

	ld de, 5
	call IncreaseScore
	
	call IncreaseNukes

	jr NoBookBonus

CheckForQuestionMark:

	cp 246
	jr c, NotQuestionMark

;The player got a question mark, so collect the correct English
;word, from which the player will need to find the matching
;word in the other language.

	push hl
	push de
	push bc
	push af

;When a player gets the wrong foreign word, a gruzzle is supposed
;to appear.  If, during that time, a new question is collected,
;force the new gruzzle out early.
	
	ld a, (GruzzleWarning+2)
	or a
	jr z, DoNotForceEarlyGruzzle

	push ix
	xor a

	ld (GruzzleWarning+2),a

;If the gruzzle needs to be forced out early, get to the correct
;tile where the new gruzzle needs to be located.

	ld hl, (GruzzleWarning)
	ld a, (hl)

;The gruzzle warning exchanges the gruzzle "tile" with the
;background.  Find out if the gruzzle ID is currently displayed
;on the map or not

	cp 28
	jr c, GruzzleIsInQuestion
	cp 32
	jr nc, GruzzleIsInQuestion

	ld (BadQuestion), a


GruzzleIsInQuestion:


	call PlaceGruzzle

	ld ix, (GruzzleWarning)
	call replacebackground
	pop ix

DoNotForceEarlyGruzzle:

;Save the location in the map data of where the question is

	ld (DisplayWords +1), ix

	pop af

;This is saved so that the correct English word can be
;displayed on the HUD

	ld (DisplayWords), a

;Turn all question marks into smiley faces.  The player
;must find the smiley face with the correct word in the foreign
;language


	ld hl, QuestionData
	ld b, 7
_
	ld e, (hl)
	inc hl
	ld d, (hl)

;We have the correct location of one smiley face

	inc hl

;Which of the sevel smiley faces?

	ld a, (hl)
	inc hl
	inc hl

	ld (de), a
	
	djnz -_

	call ReplaceBackground

	pop bc
	pop de
	pop hl
	jr NoBookBonus




NotQuestionMark:
CheckForSmile:



	cp 239
	jr c, NotSmile

;Replace this with tile 254.  This tile tells the map that,
;TEMPORARILY, we cannot put a question there.

	ld (ix), 254

	push hl
	push de
	push bc


;Find out where this smiley face is in the list of question data.

	ld hl, QuestionData + 2
	ld b, 7

FindSmileyLoop:

	cp (hl)
	inc hl
	jr z, FoundSmileyFaceInList
	
	inc hl
	inc hl
	inc hl
	jr FindSmileyLoop

FoundSmileyFaceInList:

;Is this the correct foreign word for the correct English word?	
	
	sub 239
	ld l, a

	ld a, (DisplayWords)

	sub 246
	cp l

	pop bc
	pop de
	pop hl

	jp nz, BadQuestioner	;Not the correct foreign word!  Oops!

;Never display this question mark again, because the player
;got the answer correct

	add a, a
	add a, a
	add a, 3
	push hl
	ld hl, QuestionData
	call Add_A_To_HL
	ld (hl), 1		;1 = Never display in this
				;level again
	pop hl

;This is necessary, but I don't remember why.  The value "20"
;is not important anymore, but it used to be important.

	ld a, 20
	ld (TillNextQuestionTimer), a



;Get rid of that one question mark



	ld ix, (DisplayWords +1)
	ld (ix), 0

;No English word needs to be displayed until a new question mark
;is chosen

	xor a
	ld (DisplayWords), a

	ld a, (QuestionsLeft)
	dec a
	ld (QuestionsLeft), a

;See if all 7 questions have been collected.  If so, let the player
;know that he can go to the door.

	or a
	jr nz, _

	ld hl, LevelCompleteText
	call CreateCustomHudString

_

	ld de, 20
	call IncreaseScore

	jp NoBookBonus



NotSmile:

	cp 223
	ret c

ItsABonusLetter:

	push af
	call ReplaceBackground

	ld iy, LettersLeft
	dec (iy)
	jr nz, NoLetterNukeBonus

;If a player collected all the letters, even if out of order,
;the player gets an extra nuke and 100 extra points.

	call IncreaseNukes

	ld de, 100
	call IncreaseScore
	ld hl, LetterBonusText
	call CreateCustomHudString



NoLetterNukeBonus:


	pop af


	sub 223

;Find out if this is the first, second, etc. letter in
;the bonus word

	ld hl, (OriginalBonusWordPointer)
	call Add_A_To_HL
	ld a, (hl)
	
_

	ld de, 5
	call IncreaseScore



;Bonus Word Pointer points to what letter the player
;is SUPPOSED to collect.  See if the player got the correct
;letter.

	ld iy, (BonusWordPointer)
	cp (iy)
	jp nz, NoBookBonus

;YES, the player got the correct letter.	

	inc iy
	ld (BonusWordPointer), iy

;Did the player get all the letters in the correct order?  Not yet?

	ld a, (iy)
	or a
	jr nz, NotEndOfBonusWord

	ld de, 500
	call IncreaseScore
	ld hl, WordBonusText
	call CreateCustomHudString
	call IncreaseNukes
	call IncreaseShieldsNextGame


NotEndOfBonusWord:
	
	ld iy, BonusBC
	inc (iy)


	jp NoBookBOnus



	halt
	jr -_

;Adds points to the players score.

IncreaseScore:

	ld iy, Score
	ld l, (iy +2)
	ld h, (iy +3)

	add hl, de

	ld (iy+2), l
	ld (iy+3), h

	ld l, (iy+0)
	ld h, (iy+1)

	ld de, 0
	adc hl, de

	ld (iy+0), l
	ld (iy+1), h

	ret




BadQuestioner:

;The player got an incorrect foreign equivalent of an English word


;Save the location on the map where the player got the incorrect
;smiley face.  This is where the player will see a warning that
;a new grizzle will appear.

	ld (GruzzleWarning), ix

;How much time to display the warning.

	ld a, 47
	ld (GruzzleWarning+2), a

;The location to place the new gruzzle when the warning is done
;displaying.

	ld hl, (MapX)
	ld (GruzzleWarning+3), hl
	ld hl, (MapY)
	ld (GruzzleWarning+5), hl



	call DisplayNewGrizzle

;Again, the value '20' used to be important, but not anymore


	ld a, 20
	ld (TillNextQuestionTimer), a

;Get rid of that one question mark

	ld ix, (DisplayWords +1)
	ld (ix), 0
	xor a
	ld (DisplayWords), a

	jp NoBookBonus






ReplaceBackground:

;After a powerup is collected, take the tile on its right to
;replace the background.


	ld a, (ix+1)
	ld (ix), a
	ret




CreateCustomHudString:

;Displays a custom string of text on the HUD.

	ld a, 30
	ld (CustomHudString), a
	ld (CustomHudString+1), hl

	ret


IncreaseNukes:

	ld a, (Nukes)
	inc a
	cp 10
	ret z
	
	ld (Nukes), a


	ret

IncreaseShieldsNextGame:

	ld a, (MoreShieldsNextGame)
	inc a
	ld (MoreShieldsNextGame), a

	ret